const Users = require("../infra/repository/usuarios.repository");
const users = new Users();
const AdminController = require("./admin.controller")
const adminController = new AdminController();
const jwt = require("jsonwebtoken");
const { getDetails } = require('../service/soap.client');


class usersController {

  constructor() {
    this.wsdlUrl =  'http://www.ultrasistemas.com.br/';
  }
  /**
   * Realiza o login de um usuário
   * @returns mensagem referente ao resultado obtido
   */
  login() {
    return function (req, res) {
      const data   = req.body;
      // const ip = req.socket.remoteAddress.startsWith("::ffff:") 
      //   ? req.socket.remoteAddress.split("::ffff:")[1]
      //   : req.socket.remoteAddress;



      const ip = req.headers['x-forwarded-for'] || req.connection.remoteAddress;


        
      users
        .login(data, ip)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.log(error);
          return res
            .status(400)
            .send({ error: `Não foi possivel realizar o login` });
        });
    };
  }


  loginUser() {
    return function (req, res) {
      const data = req.body;

    
      users
        .loginUser(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.log(error);
          return res
            .status(400)
            .send({ error: `Não foi possivel realizar o login` });
        });
    };
  }


  /**
   * Registra um usuário
   * @returns mensagem referente ao resultado obtido
   */
  register() {
    return function (req, res) {
      const data = req.body;

      users
        .register(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else { 
            adminController.list();
            return res.status(200).json(response);
          }
        })
        .catch((error) => {
          console.log(error);
          
          return res
            .status(400)
            .send({ error: `Não foi possível realizar o cadastro!` });
        });
    };
  }
  /**
   * Envia um email para recuperação de senha
   * @returns mensagem referente ao resultado obtido
   */
  recover() {
    return function (req, res) {
      const  data  = req.body;
      users
        .recover(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  /**
   * Realiza o logout do usuário
   * @returns
   */
  logout() {
    return function (req, res) {
      const auth = req.query.auth ? req.query.auth : "";
      users
        .logout(auth)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res
            .status(400)
            .send({ error: `Não foi possivel realizar o login` });
        });
    };
  }
  /**
   * Lista todos os usuarios
   * @returns mensagem referente ao resultado obtido
   */
  list() {
    return function (req, res) {
      const id  = req.query.id;

      users
        .list(id)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  listVendedorPorUserId() {
    return function (req, res) {
      const id_user = req.query.id_user;

      users
        .listVendedorPorUserId(id_user)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  listUserInfo() {
    return function (req, res) {
      const id_user  = req.query.id_user;

      users
        .listUserInfo(id_user)
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }

  listUserIds() {
    return function (req, res) {

      users
        .listUserIds()
        .then((response) => {
          if (response.error)
            res.status(response.code).send({ error: response.error });
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }


  /**
   * Delete um usuário
   * @returns mensagem referente ao resultado obtido
   */
  delete() {
    return function (req, res) {
      const { id } = req.params;

      users
        .delete(id)
        .then((response) => {
          if (response.error) res.status(response.code).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }
  /**
   * Atualiza os dados de um usuário
   * @returns mensagem referente ao resultado obtido
   */
  update() {
    return function (req, res) {
      const  data  = req.body;

      users
        .update(data)
        .then((response) => {
          if (response.error) res.status(400).send(response);
          else return res.status(200).send(response);
        })
        .catch((error) => {
          throw new Error(error);
        });
    };
  }


  listUserDataSocket(id_user) {
    return new Promise(async (resolve, reject) => {  


      db.query(
        `SELECT * FROM user WHERE user.id = ?`,
        [id_user],
        async (error, user) => {

          if (error) return reject(new Error(error));
          return resolve(user);
        }
      );
    });
  }



  /**
   * Verifica o código enviado por email
   * @returns mensagem referente ao resultado obtido
   */
  verify() {
    return function (req, res) {
      const { code } = req.params;
      users
        .verify(code)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res.status(400).send({ error: `Não foi possível verificar!` });
        });
    };
  }

  ultra() {
    return function (req, res) {
        const data = req.body;

        users.ultra(data).then((response) => {
            if (response.error) res.status(400).send(response);
            else return res.status(200).send(response);
        }).catch((error) => {
            throw new Error(error);
        });
    }
}

  changePassword() {
    return function (req, res) {
      const  data = req.body;
      const id_user = req.query.id_user;

      users
        .changePassword(data, id_user)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res.status(400).send({ error: `Não foi possível alterar!` });
        });
    };
  }

  listCoordenadores() {
    return function (req, res) {
      users
        .listCoordenadores()
        .then((response) => {
          if (response.error) return res.status(response.code).send(response);
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res.status(400).send({ error: `Não foi possivel alterar!` });
        });
    };
  }

  createClient() {
    return function (req, res) {
      users
        .createClient() 
        .then((response) => {
          if (response.error) return res.status(response.status).send(response);
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.error("ERRO!!! ",error);
          
          return res.status(400).send({ error: `Não foi possivel alterar!` });
        });
    };
  }

  getParceiro() {
    return function (req, res) {
      users
        .getParceiro() 
        .then((response) => {
          if (response.error) return res.status(response.status).send(response);
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.error("ERRO!!! ",error);
          
          return res.status(400).send({ error: `Não foi possivel alterar!` });
        });
    };
  }

  getPrecos() {
    return function (req, res) {
      users
        .getPrecos() 
        .then((response) => {
          if (response.error) return res.status(response.status).send(response);
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.error("ERRO!!! ",error);
          
          return res.status(400).send({ error: `Não foi possivel alterar!` });
        });
    };
  }

  saveAllOrders() {
    return function (req, res) {
      const id_user = req.query.id_user;

      users
        .saveAllOrders(id_user) 
        .then((response) => {
          if (response.error) return res.status(response.status).send(response);
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.error("ERRO!!! ",error);
          
          return res.status(400).send({ error: `Não foi possivel alterar!` });
        });
    };
  }

  getProdutos() {
    return function (req, res) {
      users
        .getProdutos() 
        .then((response) => {
          if (response.error) return res.status(response.status).send(response);
          else return res.status(200).json(response);
        })
        .catch((error) => {
          console.error("ERRO!!! ",error);
          
          return res.status(400).send({ error: `Não foi possivel alterar!` });
        });
    };
  }
  // createClient() {
  //   return async function (req, res) {
  //     try {
  //       const response = await users.createClient();
  
  //       if (response.error) {
  //         // Verifique se "response.code" é válido
  //         const statusCode = response.code || 400;
  //         return res.status(statusCode).send({
  //           error: response.error,
  //           message: response.message || "Erro ao criar cliente.",
  //         });
  //       }
  
  //       return res.status(200).json(response);
  
  //     } catch (error) {
  //       // Logando o erro para facilitar a depuração
  //       console.error("Erro no createClient:", error);
  
  //       return res.status(500).send({
  //         error: "Não foi possível alterar!",
  //         details: error.message, // Opcional: envie detalhes do erro
  //       });
  //     }
  //   };
  // }
  

  /**
   * Criptografa um valor
   * @returns o valor criptografado
   */
  encrypt() {
    return function (req, res) {
      const { data } = req.body;

      users
        .encrypt(data)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res
            .status(400)
            .send({ error: `Não foi possível criptografar!` });
        });
    };
  }

  /**
   * Acesso ao usuário pelo perfil DEV
   * @returns
   */
  accessUser() {
    return function (req, res) {
      const { data } = req.body;
      const token = jwt.decode(req.headers.authorization);

      users
        .accessUser(data, token)
        .then((response) => {
          if (response.error)
            return res.status(response.code).send({ error: response.error });
          else return res.status(200).json(response);
        })
        .catch((error) => {
          return res
            .status(400)
            .send({ error: `Não foi possível criptografar!` });
        });
    };
  }
}

module.exports = usersController;
