const {
    decrypt,
    encrypt,
    generateHash,
    checkHash,
} = require("../../config/auth/cryptography");
const { genereteToken } = require("../../config/auth/jwt");
const { encryptDonor, decryptDonor } = require("../../util/cryptDonor");
const db = require("../database/db");
const nodemailer = require("nodemailer");
const crypto = require("crypto");
const { response } = require("express");
const { getMaxListeners } = require("../database/db");
const path = require("path");
const bcrypt = require('bcrypt');

require("dotenv").config({ path: __dirname + "/./../../../.env" });

class distribuidoresRepository {

    constructor() {
        this.db = db;
    }

    verificarPedido(id_produto, id_user) {
        return new Promise((resolve, reject) => {
            this.db.query('SELECT * FROM pedidos WHERE id_produto = ? AND id_user = ?', [id_produto, id_user], (error, pedidos) => {
                if (error) return reject(new Error(error));
                resolve(pedidos);
            });
        });
    }

    obterTipoUsuario(idUsuario, callback) {
        const sql = 'SELECT tipo FROM usuarios WHERE id = ?';

        db.query(sql, [idUsuario], (err, results) => {
            if (err) {
                console.error('Erro ao buscar tipo de usuário:', err);
                return callback(err);
            }
            if (results.length === 0) {
                return callback(new Error('Usuário não encontrado.'));
            }

            const tipoUsuario = results[0].tipo;
            callback(null, tipoUsuario);
        });
    }

    // Função para buscar o preço do produto baseado no tipo de usuário
    obterPrecoProduto(idProduto, tipoUsuario, callback) {
        let colunaPreco;

        switch (tipoUsuario) {
            case 'premium':
                colunaPreco = 'preco_premium';
                break;
            case 'vip':
                colunaPreco = 'preco_vip';
                break;
            case 'normal':
            default:
                colunaPreco = 'preco_normal';
        }

        const sql = `SELECT ${colunaPreco} AS preco FROM produtos WHERE id = ?`;

        db.query(sql, [idProduto], (err, results) => {
            if (err) {
                console.error('Erro ao buscar preço do produto:', err);
                return callback(err);
            }
            if (results.length === 0) {
                return callback(new Error('Produto não encontrado.'));
            }

            const preco = results[0].preco;
            callback(null, preco);
        });
    }

    /**
     * Função para calcular o total (sem desconto adicional, pois o preço já é ajustado pelo tipo de usuário)
     * @param {*} preco 
     * @param {*} quantidade 
     * @returns 
     */
    calcularTotal(preco, quantidade) {
        return preco * quantidade;
    }


    /**
     * Registra um pedido se não houver no banco, caso haja no banco só incrementa a quantidade
     * @param {*} data dados do pedido
     * @returns mensagem referente ao resultado obtido
     */
    register(data, id_user) {
        return new Promise(async (resolve, reject) => {
            console.log(data);

            const hashedPassword = await bcrypt.hash(data.senha, 10);

                this.db.query(
                    `INSERT INTO distribuidores (razao_social, telefone, email, documento, endereco, senha) VALUES (?,?,?,?,?,?)`,
                    [data.razao_social, data.telefone, data.email, data.documento, data.endereco, hashedPassword],
                    (error) => {
                        if (error) {
                            console.error("Error inserting order:", error);
                            return reject(error);
                        }
                    }
                )


  
            return resolve({ success: "distribuidor cadastrado com sucesso!", code: 200 });
        });
    }


    /**
     * Lista todos os distribuidores
     * @returns lista com todos os dados de um distribuidor
     */
    list(id) {
        return new Promise(async (resolve, reject) => {
            var query = id
                ? `WHERE id_user = ?`
                : ``

            try {
                this.db.query(`SELECT * FROM distribuidores ${query}`, [id ? id : ""],
                    async (error, pedidos) => {

                        if (error) {

                            console.log(error);
                            return reject(new Error(error));
                        }

                        return resolve(pedidos);
                    });
            } catch (error) {
                return reject(new Error(error));
            }
        });
    }

    /**
     * Deleta um distribuidor
     * @param {*} id Id do distribuidor a ser excluído
     * @returns mensagem referente ao resultado obtido
     */
    delete(id) {
        return new Promise((resolve, reject) => {

            this.db.query('DELETE FROM distribuidores WHERE id = ?', [id],
                (error, response) => {
                    if (error) {
                        return resolve({ error: "Esta pedidos possui vinculos!", code: 203 });
                    }
                    else {
                        return resolve(true);
                    }
                });
        });
    }


    /**
     * Atualiza os dados de um distribuidor
     * @param {*} data dados atualizados da unidade
     * @returns mensagem referente ao resultado obtido
     */
    update(data) {
        return new Promise(async (resolve, reject) => {
            this.db.query(
                "UPDATE distribuidores SET documento=?, endereco=?, razao_social=?, telefone=?, email=? WHERE id = ?",
                [data.documento, data.endereco, data.razao_social, data.telefone, data.email, data.id],
                (error, response) => {
                    if (error) return reject(new Error(error));
                    else return resolve({ success: "Pedido atualizado com sucesso!" });
                }
            );
        });
    }




}

module.exports = distribuidoresRepository;
